#include <iso646.h> // boolean key words: not, and, or, xor ...
#include "cstrlist.h"

static void filedlg_collect_places_pidls(FileOpenDlgInfos *fodInfos)
{
    static const int default_places[] =
    {
#ifdef __REACTOS__
        CSIDL_RECENT,
        CSIDL_DESKTOP,
        CSIDL_MYDOCUMENTS,
        CSIDL_DRIVES,
        CSIDL_NETWORK,
#else        
        CSIDL_DESKTOP,
        CSIDL_MYDOCUMENTS,
        CSIDL_DRIVES,
#endif
    };
    unsigned int i;
    HKEY hkey;

    if (not RegOpenKeyA(HKEY_CURRENT_USER, "Software\\Microsoft\\Windows\\CurrentVersion\\Policies\\Comdlg32\\Placesbar",
            &hkey))
    {
        WCStrList pathList; // temporary list for paths, to fix duplicates problem
        
        wcstrlist_create(&pathList, (MAX_PATH + 1) * ARRAY_SIZE(fodInfos->places));
        
        // fill string list without duplicates
        for (i = 0; i < ARRAY_SIZE(fodInfos->places); i++)
        {
            WCHAR placeW[] = {'P','l','a','c','e','0',0};
            WCHAR *number = placeW + 5; // pointer of digit
            DWORD value;
            HRESULT hr;
            WCHAR *str;

            if (get_config_key_dword(hkey, placeW, &value))
            {
                WCHAR path[MAX_PATH];
                if(SHGetSpecialFolderPathW(NULL, path, value, FALSE))
                {
                    wcstrlist_append_unique_i(&pathList, path);
                }
                else
                {
                    WARN("Unrecognized special folder %u.\n", value);
                }
            }
            else if (get_config_key_string(hkey, placeW, &str))
            {
                wcstrlist_append_unique_i(&pathList, str);
                heap_free(str);
            }
            *number = *number + 1;   // set next place number; this is a replacement for swprintf
        }

        for(str = pathList.begin; *str and (str < pathList.end); )
        {
            hr = SHParseDisplayName(str, NULL, &fodInfos->places[i], 0, NULL);
            if (FAILED(hr))
                WARN("Failed to parse custom places location, %s.\n", debugstr_w(str));
            str = wcstrlist_next(&pathList, str); 
        }

        wcstrlist_destroy(&pathList);
        RegCloseKey(hkey);
        return;
    }

    for (i = 0; i < ARRAY_SIZE(default_places); i++)
        SHGetSpecialFolderLocation(NULL, default_places[i], &fodInfos->places[i]);
}
